function experiment3
%This function carries out experiment (3) in the paper.


%Read in the tree
traitfile = 'EFNdata/EFN_all_trait.txt';
treefile = 'EFNdata/EFNtree.tre'; 
[T,C] = readEFNdata(traitfile,treefile);
ntax = size(C,1);


%Find a maximum likelihood estimate of mu_r given a diffuse root.
theta.sigma2 = 1.0;
theta.sigma2_r = theta.sigma2 * treeHeight(T);
theta.mu_r = fminsearch(@(mu_r) - runLikelihood(mu_r,theta,T,C,200),-5);

N = 700;
%calculate posterior probabilities.
[~,Ps] = thresholdPosterior(T,C,theta,N);


pos = Ps(1).array; %posterior densitites
P = Ps(2).array; %posterior probabilities
X = Ps(3).array; %liability values used in the calculations.

%back-up
save ExperimentOutput/EFN_posteriors theta pos P X

%saves a nexus file with a Newick tree where each node is labelled 0, 1, or
%2, depending on the probability that the node's liability is positive.
data = findData(T.nnodes,T,C,theta,P);
fid = fopen('ExperimentOutput/posteriorTree.nex', 'w');
fprintf(fid,'#NEXUS\n\nbegin taxa;\ndimensions ntax=%d;taxlabels', T.ntax);
for n=1:ntax
    fprintf(fid,' %s', T.labels{n});
end
fprintf(fid,';\nend;\n\nbegin trees;\ntree one = [&R] %s;end;',data);
fclose(fid);

end

function height = treeHeight(T)
%Compute height of the tree (assumed ultrametric)
x = 1;
height = 0;
ntax = size(T.labels,1);
while x<2*ntax-1
    height = height + T.t(x);
    x = T.parent(x);
end
end

function LogL = runLikelihood(mu_r,theta,T,C,N)
theta.mu_r = mu_r;
LogL = thresholdGauss(T,C,theta,N);
end


function data = findData(i,T,C,theta,P)
    %uses recursion to form the Newick tree with 0, 1, and 2 as node
    %labels.
    if i <= T.ntax %leaf node
        data = strcat(num2str(2*C(i,1)),':',num2str(T.t(i)));
    elseif i < T.nnodes %internal node
        if abs(P(i)-.5) <= .2
            point = 1;
        else
            point = (P(i) > .7)*2;
        end
        data1 = findData(T.children(i,1),T,C,theta,P);
        data2 = findData(T.children(i,2),T,C,theta,P);
        data = strcat('(',data1,',',data2,')',num2str(point),':',num2str(T.t(i)));
    else %root node
        if abs(P(i)-.5) <= .2
            point = 1;
        else
            point = (P(i) > .7)*2;
        end
        data1 = findData(T.children(i,1),T,C,theta,P);
        data2 = findData(T.children(i,2),T,C,theta,P);
        data = strcat('(',data1,',',data2,')',num2str(point));
    end
end